//-------------------------------------------------------------------------------------------------
//  mcPi.c
//
//  This demonstration program computes a Monte Carlo estimation of Pi by generating pairs of
//  random numbers X and Y between 0.0 and 1.0 inclusive and then for each generated pair of X,Y
//  points computing the distance of the point from the origin.  If the distance is less than or
//  equal to 1.0, it increments a "PI pair" counter.  It does this for a specified quantity of
//  iterations.  When complete, the program estimates PI by dividing the PI-pair count by the
//  iterations count and multiplying by four, after which it outputs the computed estimate of the
//  value Pi.
//
//  This program has two required positional parameters (random seed and task ID) that together
//  "seed" the random number generator and one optional positional parameter, which is the number
//  of iterations it should perform.  The default is 1 million iterations.
//
//  This program outputs the task id, iteration count, and computed PI estimate to standard output.
//
//  Compilation command line: gcc -O2 -lm -o mcPi mcPi.c
//-------------------------------------------------------------------------------------------------
//  Date Written: 06 Apr 2015
//  Written by:   Gary D. Brown
//  Company:      Adaptive Computing Enterprises, Inc.
//-------------------------------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <math.h>


//-------------------------------------------------------------------------------------------------
//  Type definitions
//-------------------------------------------------------------------------------------------------
typedef unsigned int        uint;

//-------------------------------------------------------------------------------------------------
//  Global Constants
//-------------------------------------------------------------------------------------------------
#define FALSE               0
#define TRUE                1

//  Task ID limit
#define MAX_TASKID          10000000

//  Interactions count limits and default
#define MIN_ITERATIONS      1000
#define MAX_ITERATIONS      2100000000
#define DEFAULT_ITERATIONS  1000000


//-------------------------------------------------------------------------------------------------
//  Global variables
//-------------------------------------------------------------------------------------------------
char*   programName     = NULL;     // path to executable file

uint    random_z1       = 1;        // used by random number generator
uint    random_z2       = 2;        // used by random number generator
uint    random_z3       = 3;        // used by random number generator
uint    random_z4       = 4;        // used by random number generator



//-------------------------------------------------------------------------------------------------
//  displayUsage
//
//  This routine displays an optional error message and then displays the usage message.
//-------------------------------------------------------------------------------------------------
void displayUsage(char* message)
{
    if (message != NULL)
    {
        fprintf(stderr, "\n");
        fprintf(stderr, "*** %s\n", message);
        fprintf(stderr, "\n");
    }
    fprintf(stderr, "Usage: %s <seed> <task> [<iter>]\n", programName);
    fprintf(stderr, "seed = required random number generator seed value (e.g. $SEED)\n");
    fprintf(stderr, "task = required task ID (e.g. $NITROTASKID)\n");
    fprintf(stderr, "iter = optional count of interations to perform (default %d)\n", DEFAULT_ITERATIONS);
    fprintf(stderr, "\n");
    fprintf(stderr, "To pass the same seed value to every task instantiation, create a seed value\n");
    fprintf(stderr, "in your job script and export the value in an environment variable (e.g. $SEED)\n");
    fprintf(stderr, "you can place in each Nitro task definition.  This program will add the seed\n");
    fprintf(stderr, "and Nitro task ID values together for the actual seed value used to generate\n");
    fprintf(stderr, "random numbers so each task has a different seed value.\n");
    fprintf(stderr, "\n");
    fflush(stderr);
}

//-------------------------------------------------------------------------------------------------
//  parseParms
//
//  This routine parses the command line.
//-------------------------------------------------------------------------------------------------
int parseParms(int     argc,
               char**  argv,
               uint*   seed,
               int*    taskID,
               int*    iterations)
{
char*   parm_ptr    = NULL;
char    message_buffer[1024];

        // parse required seed parameter

    if (--argc == 0)            // missing required parameter
    {
        displayUsage("Missing required seed parameter value");
        return FALSE;
    }
    parm_ptr = *argv++;
    *seed = atoi(parm_ptr);
    if (*seed <= 0)
    {
        sprintf(message_buffer, "Invalid seed value (%s)-- must be positive integer", parm_ptr);
        displayUsage(message_buffer);
        return FALSE;
    }

        // parse required taskID parameter

    if (--argc == 0)            // missing required parameter
    {
        displayUsage("Missing required taskID parameter value");
        return FALSE;
    }
    parm_ptr = *argv++;
    *taskID = atoi(parm_ptr);
    if (*taskID <= 0)
    {
        sprintf(message_buffer, "Invalid taskID value (%s)-- must be a positive integer <= %d",
            parm_ptr, MAX_TASKID);
        displayUsage(message_buffer);
        return FALSE;
    }

        // Parse optional iterations parameter

    if (--argc == 0)            // optional parameter not present
        *iterations = DEFAULT_ITERATIONS;
    else                        // parameter present
    {
        parm_ptr = *argv++;
        *iterations = atoi(parm_ptr);
        if ((*iterations < MIN_ITERATIONS) || (*iterations > MAX_ITERATIONS))
        {
            sprintf(message_buffer, "Invalid iterations count (%s)-- must be in range %d-%d",
                parm_ptr, MIN_ITERATIONS, MAX_ITERATIONS);
            displayUsage(message_buffer);
            return FALSE;
        }
    }

        // Check for too many positional parameters
    if (--argc > 0)
    {
        displayUsage("Too many command line parameters");
        return FALSE;
    }

        // Success
    return TRUE;
}


//-------------------------------------------------------------------------------------------------
//  setRandom
//
//  This routine sets the "seed value" of the random number generator.
//-------------------------------------------------------------------------------------------------
void setRandom(uint seed)
{
    random_z1 = (seed *  3) * 1099087573UL;
    random_z2 = (seed *  5) * 1099087573UL;
    random_z3 = (seed *  7) * 1099087573UL;
    random_z4 = (seed * 11) * 1099087573UL;
}


//-------------------------------------------------------------------------------------------------
//  tauswortheRNG
//
//  Tausworthe random number generator
//
//  see 2.4 in http://iopscience.iop.org/1742-6596/368/1/012024/pdf/1742-6596_368_1_012024.pdf
//-------------------------------------------------------------------------------------------------
uint tauswortheRNG(       uint* z,
                    const int   s1,
                    const int   s2,
                    const int   s3,
                    const uint  m)
{
    return *z = ((*z & m) << s3) ^ (((*z << s1) ^ *z) >> s2);
}


//-------------------------------------------------------------------------------------------------
//  lcgRNG
//
//  Linear congruential generator
//-------------------------------------------------------------------------------------------------
uint lcgRNG(      uint*    z,
            const uint     a,
            const uint     c)
{
    return *z = (a * *z) + c;
}


//-------------------------------------------------------------------------------------------------
//  hybridTauswortheRNG
//
//  This Tausworthe random number generator returns a value in the range [0..4294967295] using a
//  hybrid random number generator composed of three Tausworthe generators combined with a linear
//  congruential generator (LCG) suggested by Nvidia.  It has a period of approximately 2^121.
//
//  see 3 in http://iopscience.iop.org/1742-6596/368/1/012024/pdf/1742-6596_368_1_012024.pdf
//-------------------------------------------------------------------------------------------------
uint hybridTauswortheRNG()
{
    return tauswortheRNG( &random_z1, 13, 19, 12, 4294967294UL) ^  // period 1 = 2^31 - 1
           tauswortheRNG( &random_z2,  2, 25,  4, 4294967288UL) ^  // period 2 = 2^30 - 1
           tauswortheRNG( &random_z3,  3, 11, 17, 4294967280UL) ^  // period 3 = 2^28 - 1
           lcgRNG(        &random_z4, 1664525, 1013904223UL);      // period 4 = 2^32
}


//-------------------------------------------------------------------------------------------------
//  getUnitRandom
//
//  Converts the random integer in the range [0..4294967295] into the real range [0.0 .. 1.0].
//-------------------------------------------------------------------------------------------------
double getUnitRandom()
{
    return ((double) 1.0 / (double) 4294967295) * hybridTauswortheRNG();
//  return  hybridTauswortheRNG() / (double) 4294967295;
}


//-------------------------------------------------------------------------------------------------
//  main
//
//  This routine parses the command line, performs the Monte Carlo method for estimating PI, and
//  then outputs the estimated PI value.
//-------------------------------------------------------------------------------------------------
int main(int argc, char** argv)
{
int     seed                = 0;
int     taskID              = 0;
int     iterationsCount     = 0;
int     iterations          = 0;
int     withinCircleCount   = 0;
double  x                   = 0.0;
double  y                   = 0.0;
double  pi                  = 0.0;

    //  Save the program's path
    programName = *argv++;

    //  Parse the command line
    if (!parseParms(argc, argv, &seed, &taskID, &iterationsCount))
    {
        return 1;
    }

    //  Estimate PI via Monte Carlo method
    setRandom(seed + taskID);       // seed the random number generator
    while (++iterations <= iterationsCount)
    {
        x = getUnitRandom();
        y = getUnitRandom();
        if (sqrt((x * x) + (y * y)) <= 1.0)
        {
            ++withinCircleCount;
        }
    }
    // Calculate PI estimation
    pi = (4.0 * (double) withinCircleCount) / (double) iterationsCount;

    //  Output PI estimate
    printf("%ld %ld %.9lf\n", taskID, iterationsCount, pi);
    fflush(stdout);

    return 0;
}
