#!/bin/bash

#==========================================================================================
#  THIS SOFTWARE SCRIPT IS PROVIDED BY ADAPTIVE COMPUTING ENTERPRISES, INC. "AS IS" AND IS
#  FOR REFERENCE USE ONLY.  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
#  TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED.  IN NO EVENT SHALL ADAPTIVE COMPUTING ENTERPRISES, INC. BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES ARISING FROM
#  THE USE THEREOF.
#
#  This script will launch a Nitro job from a SLURM-supported Cray ALPS environment.
#  It will run the Nitro coordinator on the first host in the SLURM_JOB_NODELIST and Nitro
#  workers on all of the other unique hosts listed in the file.
#
#  This script does not accept any arguments but does allow control of Nitro's execution
#  via environment variables.  It supports the following environment variables, which a
#  user can set in a job script.  This "launch" script can augment or manipulate the
#  values of the environment variables to suit the site's needs and policies.
#
#  NITRO_OPTIONS        : Options passed on the command line to workers and coordinators.
#  NITRO_WORKER_OPTIONS : Options passed on the command line to workers only.
#  NITRO_COORD_OPTIONS  : Options passed on the command line to coordinators only.  An
#                         example is the "--run-local-worker" option.
#  NITROJOBID           : If present, this is used by --job-id in NITRO_OPTIONS; otherwise,
#  SLURM_JOB_ID         : If present, this is used by --job-id in NITRO_OPTIONS
#
#  FIRST NOTE TO ADMINISTRATOR:
#  Modify this script as needed for your site and write the modified version to the Nitro
#  "bin" directory (default: /opt/nitro/bin) from where users' job scripts can execute it.
#
#  SECOND NOTE TO ADMINISTRATOR:
#  DO NOT MODIFY THE ORIGINAL ADAPTIVE-SUPPLIED SCRIPT IN THE "<installDir>/scripts" SUB-
#  DIRECTORY AND THEN HAVE USER JOB SCRIPTS EXECUTE IT BECAUSE NITRO UPDATES WILL OVERWRITE
#  YOUR MODIFICATIONS.  The "*/scripts" sub-directories are for the factory versions.
#==========================================================================================

#------------------------------------------------------------------------------------------
#  NOTE TO ADMINISTRATOR
#
#  If Nitro is installed anywhere other than the default location of /opt/nitro, set its
#  location here.
#------------------------------------------------------------------------------------------
NITRO=/opt/nitro/bin/nitro

#------------------------------------------------------------------------------------------
#  Verify the Nitro program is present and executable.
#------------------------------------------------------------------------------------------
if [ ! -f ${NITRO} ]
then
    echo "Error: Nitro program file path is incorrect."
    echo "* $0 contains \$NITRO variable with non-existent file path ($NITRO)."
    echo "* Nitro variable must contain a valid file path before Nitro jobs will execute."
    echo "* Aborting Nitro job."
    exit 1
fi
if [ ! -x ${NITRO} ]
then
    echo "Error: Nitro program file permissions are incorrect."
    echo "* Cannot execute Nitro program ($NITRO) due to insufficient permissions."
    echo "* Administrator must assign valid permissions to Nitro program file."
    echo "* Aborting Nitro job."
    exit 1
fi

#------------------------------------------------------------------------------------------
#  Check coordinator host.  If the coordinator host is not on the default port (47000)
#  then the coordinator host should be specified as <host>:<port>, or the --port-file
#  option can be used to cause the workers to read the port file for the coordinator port.
#------------------------------------------------------------------------------------------
if [ -z ${NITROCOORDHOST+x} ]
then
    PORT_FILE_FLAG=false
    if [ ! -z ${NITRO_WORKER_OPTIONS+x} ]
    then
        if [[ $NITRO_COORD_OPTIONS == *"--port-file"* ]]
        then
            PORT_FILE_FLAG=true
        fi
    else
        if [[ $NITRO_OPTIONS == *"--port-file"* ]]
        then
            PORT_FILE_FLAG=true
        fi
    fi

    if ! $PORT_FILE_FLAG
    then
        echo "Error: Nitro coordinator host must be supplied to workers, or the --port-file flag used."
        echo "* Job must be submitted with NITROCOORDHOST set to host of coordinator job or the"
        echo "* --port-file flag must be used to indicate the the workers should use the jobs"
        echo "* coordinator port file to locate the coordinator node."
        echo "* Aborting Nitro job."
        exit 1
    fi
else
    NITRO_OPTIONS="--coord ${NITROCOORDHOST} ${NITRO_OPTIONS} "
fi

#------------------------------------------------------------------------------------------
#  Set up the job id
#
#  If the user has set the NITROJOBID environment variable for the job, use it.
#  Otherwise, try to use the scheduler-provided job id.
#  If there is no scheduler-provided job id, generate a timestamp-based job id.
#------------------------------------------------------------------------------------------
if [ -z ${NITROJOBID+x} ]
then
    echo "Error: Nitro job id must be supplied to workers."
    echo "* Job must be submitted with NITROJOBID set to job id of coordinator job."
    echo "* Aborting Nitro job."
    exit 1
fi

#------------------------------------------------------------------------------------------
#  Set the job id in the command line options for the Nitro coordinator and workers.
#------------------------------------------------------------------------------------------
NITRO_OPTIONS="--job-id ${NITROJOBID} ${NITRO_OPTIONS}"

#------------------------------------------------------------------------------------------
#  The workers must have the same session key at the coordinator job
#------------------------------------------------------------------------------------------
if [ -z $NITROSESSIONKEY ]
then
    echo "Error: Nitro session key must be supplied to workers."
    echo "* Job must be submitted with NITROSESSIONKEY set to session key of coordinator job."
    echo "* Aborting Nitro job."
    exit 1
fi

#------------------------------------------------------------------------------------------
#  Verify that the SLURM_JOB_NODELIST environment variable contains a lists of host names
#  and get the host name of the first host for use as the Nitro coordinator.
#------------------------------------------------------------------------------------------
if [ -z ${SLURM_JOB_NODELIST+x} ]
then
    echo "Error: Missing SLURM_JOB_NODELIST environment variable error."
    echo "* $0 did not receive the required \$SLURM_JOB_NODELIST environment variable supplied"
    echo "* by SLURM. Please contact your administrator to correct this configuration"
    echo "* issue before Nitro jobs will execute."
    exit 1
fi
if [ ! -f ${SLURM_JOB_NODELIST} ]
then
    echo "Error: SLURM_JOB_NODELIST contains a path that does not exist."
    echo "* \$SLURM_JOB_NODELIST contains non-existent file path ($SLURM_JOB_NODELIST)"
    echo "* Please contact your administrator must correct this configuration"
    echo "* issue before Nitro jobs will execute."
    exit 1
fi

#------------------------------------------------------------------------------------------
#  Cray systems require the depth to be set appropriately to the system. If depth is not
#  set, then nodes will only receive one core.
#
#  For Cray XE5 systems, depth cannot exceed 12.
#  For Cray XK6 compute blades, depth cannot exceet 16.
#  For Cray XE6 systems, depth cannot exceed 32
#
#  If the SLURM_JOB_CPUS_PER_NODE value is set then use that value, otherwise we need a
#  user supplied value.
#------------------------------------------------------------------------------------------
if [ ! -z ${SLURM_JOB_CPUS_PER_NODE+x} ]
then
    NODE_DEPTH=$SLURM_JOB_CPUS_PER_NODE
else
    NODE_DEPTH=32
fi

#------------------------------------------------------------------------------------------
# Cray nodes report a hostname in the format "nidxxxxxx", but the PBS_NODEFILE just
# contains the numeric portion of the node id. To tell Nitro what nodes it is expecting
# Nitro needs to have a file of nodes in the "nidxxxxx" format instead. We also need
# a file containing the node list for the worker aprun command.  These files need to be
# placed in the job directory which is either specified, or is assumed to be in the users
# $HOME/nitro directory.
#------------------------------------------------------------------------------------------
NITRO_FILE_DIR=$NITROJOBDIR
if [ -z $NITRO_FILE_DIR ]
then
    if [ ! -f $HOME/nitro ]
    then
        mkdir $HOME/nitro
    fi
    if [ ! -f $HOME/nitro/$NITROJOBID ]
    then
        mkdir $HOME/nitro/$NITROJOBID
    fi
    NITRO_FILE_DIR=$HOME/nitro/$NITROJOBID
fi
HOSTNAME=`hostname`
NODEFILE_BASENAME=$NITRO_FILE_DIR/nodelist_$HOSTNAME
NODEFILE_APRUN=${NODEFILE_BASENAME}_aprun
NODEFILE_WORKERS=${NODEFILE_BASENAME}_workers

# setup the node file to use with aprun
scontrol show hostnames ${SLURM_JOB_NODELIST} > $NODEFILE_APRUN

# setup the workers file for the coordinator
awk '{printf "nid%.5d\n",$1}' $PBS_NODEFILE | uniq > $NODEFILE_WORKERS

if [ -z ${NITRO_COORD_OPTIONS} ]
then
    NITRO_COORD_OPTIONS="--workers-file ${NODEFILE_WORKERS}"
else
    NITRO_COORD_OPTIONS="--workers-file ${NODEFILE_WORKERS} ${NITRO_COORD_OPTIONS}"
fi

#------------------------------------------------------------------------------------------
#  Start the Nitro workers on all but the first compute node
#------------------------------------------------------------------------------------------
echo
echo "Launching Nitro workers"
aprun -b -l $NODEFILE_APRUN -n $((SLURM_JOB_NUM_NODES)) -d $NODE_DEPTH -j 0 -cc depth $NITRO --mode worker --key $NITROSESSIONKEY $NITRO_OPTIONS $NITRO_WORKER_OPTIONS

#------------------------------------------------------------------------------------------
# cleanup temporary node files
#------------------------------------------------------------------------------------------
rm -f $NODEFILE_WORKERS
rm -f $NODEFILE_APRUN
