#bin/bash

#==========================================================================================
#  THIS SOFTWARE SCRIPT IS PROVIDED BY ADAPTIVE COMPUTING ENTERPRISES, INC. "AS IS" AND IS
#  FOR REFERENCE USE ONLY.  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
#  TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED.  IN NO EVENT SHALL ADAPTIVE COMPUTING ENTERPRISES, INC. BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES ARISING FROM
#  THE USE THEREOF.
#
#  This script will launch a Nitro job from a TORQUE-supported Cray ALPS environment.
#  It will run the Nitro coordinator on the first host in the PBS_NODEFILE file and Nitro
#  workers on all of the other unique hosts listed in the file.
#
#  This script does not accept any arguments but does allow control of Nitro's execution
#  via environment variables.  It supports the following environment variables, which a
#  user can set in a job script.  This "launch" script can augment or manipulate the
#  values of the environment variables to suit the site's needs and policies.
#
#  NITRO_OPTIONS        : Options passed on the command line to workers and coordinators.
#  NITRO_WORKER_OPTIONS : Options passed on the command line to workers only.
#  NITRO_COORD_OPTIONS  : Options passed on the command line to coordinators only.  An
#                         example is the "--run-local-worker" option.
#  NITROJOBID           : If present, this is used by --job-id in NITRO_OPTIONS; otherwise,
#  PBS_JOBID            : If present, this is used by --job-id in NITRO_OPTIONS.
#
#  FIRST NOTE TO ADMINISTRATOR:
#  Modify this script as needed for your site and write the modified version to the Nitro
#  "bin" directory (default: /opt/nitro/bin) from where users' job scripts can execute it.
#
#  SECOND NOTE TO ADMINISTRATOR:
#  DO NOT MODIFY THE ORIGINAL ADAPTIVE-SUPPLIED SCRIPT IN THE "<installDir>/scripts" SUB-
#  DIRECTORY AND THEN HAVE USER JOB SCRIPTS EXECUTE IT BECAUSE NITRO UPDATES WILL OVERWRITE
#  YOUR MODIFICATIONS.  The "*/scripts" sub-directories are for the factory versions.
#==========================================================================================

#------------------------------------------------------------------------------------------
#  NOTE TO ADMINISTRATOR
#
#  If Nitro is installed anywhere other than the default location of /opt/nitro, set its
#  location here.
#------------------------------------------------------------------------------------------
NITRO=/opt/nitro/bin/nitro

#------------------------------------------------------------------------------------------
#  Verify the Nitro program is present and executable.
#------------------------------------------------------------------------------------------
if [ ! -f ${NITRO} ]
then
    echo "Error: Nitro program file path is incorrect."
    echo "* $0 contains \$NITRO variable with non-existent file path ($NITRO)."
    echo "* Nitro variable must contain a valid file path before Nitro jobs will execute."
    echo "* Aborting Nitro job."
    exit 1
fi
if [ ! -x ${NITRO} ]
then
    echo "Error: Nitro program file permissions are incorrect."
    echo "* Cannot execute Nitro program ($NITRO) due to insufficient permissions."
    echo "* Administrator must assign valid permissions to Nitro program file."
    echo "* Aborting Nitro job."
    exit 1
fi

#------------------------------------------------------------------------------------------
#  Generate a session key if one isn't specified
#
#  Using a session key allows the workers to attach to the coordinator without having
#  to be named or using a worker list
#------------------------------------------------------------------------------------------
if [ -z $NITROSESSIONKEY ]
then
    NITROSESSIONKEY=`date +%s%N | md5sum | head -c16 ; echo`
fi

#------------------------------------------------------------------------------------------
#  Verify the NITRO_TASK_FILE environment variable exists and has a readable file path.
#  However, if the linger option is specified (in either NITRO_OPTIONS or
#  NITRO_COORD_OPTIONS), then a task file is not necessary.
#------------------------------------------------------------------------------------------
if [ -z ${NITRO_TASK_FILE+x} ]
then
    if [ -z ${NITRO_LONG_TASK_FILE+x} ]
    then
        LINGER_MODE=false
        if [ ! -z ${NITRO_COORD_OPTIONS+x} ]
        then
            if [[ $NITRO_COORD_OPTIONS == *"--linger"* ]]
            then
                LINGER_MODE=true
            fi
        else
            if [[ $NITRO_OPTIONS == *"--linger"* ]]
            then
                LINGER_MODE=true
            fi
        fi

        if ! $LINGER_MODE
        then
            echo "Error: Missing NITRO_TASK_FILE environment variable."
            echo "* Expected \$NITRO_TASK_FILE or \$NITRO_LONG_TASK_FILE environment variable"
            echo "* to contain a task file name."
            echo "* Assign the Nitro task file path to the NITRO_TASK_FILE environment variable"
            echo "* and resubmit the job."
            exit 1
        fi
    fi
fi

#------------------------------------------------------------------------------------------
#  Set up the job id
#
#  If the user has set the NITROJOBID environment variable for the job, use it.
#  Otherwise, try to use the scheduler-provided job id.
#  If there is no scheduler-provided job id, generate a timestamp-based job id.
#------------------------------------------------------------------------------------------
if [ -z ${NITROJOBID+x} ]
then
    if [ ! -z ${PBS_JOBID+x} ]
    then
        NITROJOBID=${PBS_JOBID%%.*}
    else
        NITROJOBID=`date +%Y%m%d%H%M%S`
    fi
fi
#------------------------------------------------------------------------------------------
#  Set the job id in the command line options for the Nitro coordinator and workers.
#------------------------------------------------------------------------------------------
if [ -z ${NITRO_OPTIONS} ]
then
    NITRO_OPTIONS="--job-id ${NITROJOBID}"
else
    NITRO_OPTIONS="--job-id ${NITROJOBID} ${NITRO_OPTIONS}"
fi

#------------------------------------------------------------------------------------------
#  Verify that the PBS_NODEFILE environment variable contains a host names file
#  and get the host name of the first host for use as the Nitro coordinator.
#------------------------------------------------------------------------------------------
if [ -z ${PBS_NODEFILE+x} ]
then
    echo "Error: Missing PBS_NODEFILE environment variable error."
    echo "* $0 did not receive the required \$PBS_NODEFILE environment variable supplied"
    echo "* by TORQUE. Please contact your administrator to correct this configuration"
    echo "* issue before Nitro jobs will execute."
    exit 1
fi
if [ ! -f ${PBS_NODEFILE} ]
then
    echo "Error: PBS_NODEFILE contains a path that does not exist."
    echo "* \$PBS_NODEFILE contains non-existent file path ($PBS_NODEFILE)"
    echo "* Please contact your administrator must correct this configuration"
    echo "* issue before Nitro jobs will execute."
    exit 1
fi

#------------------------------------------------------------------------------------------
#  Cray systems require the depth to be set appropriately to the system. If depth is not
#  set, then nodes will only receive one core.
#
#  For Cray XE5 systems, depth cannot exceed 12.
#  For Cray XK6 compute blades, depth cannot exceet 16.
#  For Cray XE6 systems, depth cannot exceed 32
#
#  If the PBS_NUM_PPN value is set then use that value, otherwise we need a
#  user supplied value.
#------------------------------------------------------------------------------------------
if [ ! -z ${PBS_NUM_PPN+x} ]
then
    NODE_DEPTH=$PBS_NUM_PPN
else
    NODE_DEPTH=32
fi

#------------------------------------------------------------------------------------------
# Cray nodes report a hostname in the format "nidxxxxxx", but the PBS_NODEFILE just
# contains the numeric portion of the node id. To tell Nitro what nodes it is expecting
# Nitro needs to have a file of nodes in the "nidxxxxx" format instead. We also need
# a file containing the node list for the worker aprun command.  These files need to be
# placed in the job directory which is either specified, or is assumed to be in the users
# $HOME/nitro directory.
#------------------------------------------------------------------------------------------
NITRO_FILE_DIR=$NITROJOBDIR
if [ -z $NITRO_FILE_DIR ]
then
    if [ ! -f $HOME/nitro ]
    then
        mkdir $HOME/nitro
    fi
    if [ ! -f $HOME/nitro/$NITROJOBID ]
    then
        mkdir $HOME/nitro/$NITROJOBID
    fi
    NITRO_FILE_DIR=$HOME/nitro/$NITROJOBID
fi

NODEFILE_BASENAME=$NITRO_FILE_DIR/nodelist
NODEFILE_APRUN=${NODEFILE_BASENAME}_aprun
NODEFILE_WORKERS=${NODEFILE_BASENAME}_workers

# Get the coordinator node - use the first node in the node file
CNODE=`head -n1 $PBS_NODEFILE`

# Get the coordinator host name - this is the first node in "nidxxxxx" format
CHOST=`echo ${CNODE} | awk '{printf "nid%.5d\n",$1}'`

# setup the node file to use with aprun
uniq $PBS_NODEFILE | tail -n +2 > $NODEFILE_APRUN

# setup the workers file for the coordinator
awk '{printf "nid%.5d\n",$1}' $PBS_NODEFILE | uniq > $NODEFILE_WORKERS

if [ -z ${NITRO_COORD_OPTIONS} ]
then
    NITRO_COORD_OPTIONS="--workers-file ${NODEFILE_WORKERS}"
else
    NITRO_COORD_OPTIONS="--workers-file ${NODEFILE_WORKERS} ${NITRO_COORD_OPTIONS}"
fi

#------------------------------------------------------------------------------------------
#  Start the Nitro workers on all but the first compute node
#------------------------------------------------------------------------------------------
echo
echo "Launching Nitro workers"
aprun -b -N 1 -l $NODEFILE_APRUN -n $((PBS_NUM_NODES-1)) -d $NODE_DEPTH -j 0 -cc depth $NITRO --mode worker --coord $CHOST $NITRO_OPTIONS $NITRO_WORKER_OPTIONS &

#------------------------------------------------------------------------------------------
#  Start the Nitro coordinator
#------------------------------------------------------------------------------------------
echo
echo "Launching Nitro coordinator"
aprun -b -N 1 -L $CNODE -n 1 -d $NODE_DEPTH -j 0 --cc depth $NITRO --mode coord $NITRO_OPTIONS $NITRO_COORD_OPTIONS $NITRO_TASK_FILE "$NITRO_LONG_TASK_FILE" &
wait

#------------------------------------------------------------------------------------------
# cleanup temporary node files
#------------------------------------------------------------------------------------------
rm -f $NODEFILE_WORKERS
rm -f $NODEFILE_APRUN
