#!/bin/bash

#==========================================================================================
#  THIS SOFTWARE SCRIPT IS PROVIDED BY ADAPTIVE COMPUTING ENTERPRISES, INC. "AS IS" AND IS
#  FOR REFERENCE USE ONLY.  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
#  TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
#  DISCLAIMED.  IN NO EVENT SHALL ADAPTIVE COMPUTING ENTERPRISES, INC. BE LIABLE FOR ANY
#  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES ARISING FROM
#  THE USE THEREOF.
#
#  This script will launch a Nitro job from a TORQUE environment.  It will run the Nitro
#  coordinator on the first host in the PBS_NODEFILE file and Nitro workers on all of the
#  other unique hosts listed in the file.
#
#  This script does not accept any arguments but does allow control of Nitro's execution
#  via environment variables.  It supports the following environment variables, which a
#  user can set in a job script.  This "launch" script can augment or manipulate the
#  values of the environment variables to suit the site's needs and policies.
#
#  NITRO_OPTIONS        : Options passed on the command line to workers and coordinators.
#  NITRO_WORKER_OPTIONS : Options passed on the command line to workers only.
#  NITRO_COORD_OPTIONS  : Options passed on the command line to coordinators only.  An
#                         example is the "--run-local-worker" option.
#  NITROJOBID           : If present, this is used by --job-id in NITRO_OPTIONS; otherwise,
#  LSB_JOBID            : If present, this is used by --job-id in NITRO_OPTIONS
#
#  FIRST NOTE TO ADMINISTRATOR:
#  Modify this script as needed for your site and write the modified version to the Nitro
#  "bin" directory (default: /opt/nitro/bin) from where users' job scripts can execute it.
#
#  SECOND NOTE TO ADMINISTRATOR:
#  DO NOT MODIFY THE ORIGINAL ADAPTIVE-SUPPLIED SCRIPT IN THE "<installDir>/scripts" SUB-
#  DIRECTORY AND THEN HAVE USER JOB SCRIPTS EXECUTE IT BECAUSE NITRO UPDATES WILL OVERWRITE
#  YOUR MODIFICATIONS.  The "*/scripts" sub-directories are for the factory versions.
#==========================================================================================

#------------------------------------------------------------------------------------------
#  NOTE TO ADMINISTRATOR
#
#  If Nitro is installed anywhere other than the default location of /opt/nitro, set its
#  location here.
#------------------------------------------------------------------------------------------
NITRO=/opt/nitro/bin/nitro

#------------------------------------------------------------------------------------------
#  Verify the Nitro program is present and executable.
#------------------------------------------------------------------------------------------
if [ ! -f ${NITRO} ]
then
    echo "Error: Nitro program file path is incorrect."
    echo "* $0 contains \$NITRO variable with non-existent file path ($NITRO)."
    echo "* Nitro variable must contain a valid file path before Nitro jobs will execute."
    echo "* Aborting Nitro job."
    exit 1
fi
if [ ! -x ${NITRO} ]
then
    echo "Error: Nitro program file permissions are incorrect."
    echo "* Cannot execute Nitro program ($NITRO) due to insufficient permissions."
    echo "* Administrator must assign valid permissions to Nitro program file."
    echo "* Aborting Nitro job."
    exit 1
fi

#------------------------------------------------------------------------------------------
#  Set up the job id
#
#  If the user has set the NITROJOBID environment variable for the job, use it.
#  Otherwise, try to use the scheduler-provided job id.
#  If there is no scheduler-provided job id, generate a timestamp-based job id.
#------------------------------------------------------------------------------------------
if [ -z ${NITROJOBID+x} ]
then
    if [ ! -z ${LSB_JOBID+x} ]
    then
        NITROJOBID=${LSB_JOBID}
    else
        NITROJOBID=`date +%Y%m%d%H%M%S`
    fi
fi

#------------------------------------------------------------------------------------------
#  Set the job id in the command line options for the Nitro coordinator and workers.
#------------------------------------------------------------------------------------------
NITRO_OPTIONS="--job-id ${NITROJOBID} ${NITRO_OPTIONS}"

#------------------------------------------------------------------------------------------
#  Generate a session key if one isn't specified
#
#  Using a session key allows the workers to attach to the coordinator without having
#  to be named or using a worker list
#------------------------------------------------------------------------------------------
if [ -z $NITROSESSIONKEY ]
then
    NITROSESSIONKEY=`date +%s%N | md5sum | head -c16 ; echo`
fi

#------------------------------------------------------------------------------------------
#  Verify the NITRO_TASK_FILE environment variable exists and has a readable file path.
#  However, if the linger option is specified (in either NITRO_OPTIONS or
#  NITRO_COORD_OPTIONS), then a task file is not necessary.
#------------------------------------------------------------------------------------------
if [ -z ${NITRO_TASK_FILE+x} ]
then
    if [ -z ${NITRO_LONG_TASK_FILE+x} ]
    then
        LINGER_MODE=false
        if [ ! -z ${NITRO_COORD_OPTIONS+x} ]
        then
            if [[ $NITRO_COORD_OPTIONS == *"--linger"* ]]
            then
                LINGER_MODE=true
            fi
        else
            if [[ $NITRO_OPTIONS == *"--linger"* ]]
            then
                LINGER_MODE=true
            fi
        fi

        if ! $LINGER_MODE
        then
            echo "Error: Missing NITRO_TASK_FILE environment variable."
            echo "* Expected \$NITRO_TASK_FILE or \$NITRO_LONG_TASK_FILE environment variable"
            echo "* to contain a task file name."
            echo "* Assign the Nitro task file path to the NITRO_TASK_FILE environment variable"
            echo "* and resubmit the job."
            exit 1
        fi
    fi
fi

#------------------------------------------------------------------------------------------
#  Verify that the LSB_HOSTS environment variable contains a list of host names
#------------------------------------------------------------------------------------------
if [ -z ${LSB_HOSTS+x} ]
then
    echo "Error: Missing LSB_HOSTS environment variable error."
    echo "* $0 did not receive the required \$LSB_HOSTS environment variable supplied"
    echo "* by LSB. Please contact your administrator to correct this configuration"
    echo "* issue before Nitro jobs will execute."
    exit 1
fi

#------------------------------------------------------------------------------------------
#  Get an array of unique hostnames, keeping the first as the coordinator
#------------------------------------------------------------------------------------------
NODES=(${LSB_HOSTS})
COORD_HOST=${NODES[0]}
WORKERS=( $(tr ' ' '\n' <<< ${NODES[*]} | grep -v "^${COORD_HOST}$" | sort -u) )

#------------------------------------------------------------------------------------------
#  Start the Nitro workers on all but the first compute node
#------------------------------------------------------------------------------------------
echo
echo "Launching Nitro workers"
for WORKER in ${WORKERS[*]};
do
    blaunch ${WORKER} ${NITRO} --mode=worker --coord ${COORD_HOST} --name ${WORKER} --key $NITROSESSIONKEY ${NITRO_OPTIONS} ${NITRO_WORKER_OPTIONS} &
done

#------------------------------------------------------------------------------------------
#  Start the Nitro coordinator with exec so the coordinator will receive
#  any signals sent by the resource manager.
#------------------------------------------------------------------------------------------
echo "Launching Nitro coordinator"
echo
EXECHOSTS=$(IFS=+ ; echo "${WORKERS[*]}")
exec ${NITRO} --mode=coord --workers ${EXECHOSTS} --key $NITROSESSIONKEY $NITRO_OPTIONS $NITRO_COORD_OPTIONS $NITRO_TASK_FILE "$NITRO_LONG_TASK_FILE"
